/**
  * @file BrushlessARM32.c
  * @brief This module contains the implementation of the brushless hardware
  * functions.

  * @author Stefan Popovic
  *
  * @date 11/09/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: STM32F107VC
  * Dev.Board: EasyMx PRO v7
  * Oscillator: HS-PLL 72 MHz
  * Ext. Modules: Brushless Click Board
  * SW: MikroC PRO for ARM v4.9.0

  * NOTES:
    - Place Brushless click board on mikroBUS slot 1
    - Turn on PORTA LEDs at SW15, pull-down PORTD.3 and PORTD.6 and turn on Button Press Level switch SW10 for PORTB/L. (board specific)
    - PWM: PA0  TIM5_CH1
    - Vsense: PA4    ADC12_I4
    - Int: PD10

  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/
  #define CW    0x01
  #define CCW   0x00
  #define STEP  50

  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/


  /**************************************************************************
  * Module Typedefs
  **************************************************************************/
  

  /**************************************************************************
  * Includes
  **************************************************************************/
  #include "built_in.h"
  #include "recources.h"

  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/
  sbit MOTOR_DIR at GPIOC_ODR.B2;
  
  // TFT module connections
  unsigned int TFT_DataPort at GPIOE_ODR;
  sbit TFT_RST at GPIOE_ODR.B8;
  sbit TFT_RS at GPIOE_ODR.B12;
  sbit TFT_CS at GPIOE_ODR.B15;
  sbit TFT_RD at GPIOE_ODR.B10;
  sbit TFT_WR at GPIOE_ODR.B11;
  sbit TFT_BLED at GPIOE_ODR.B9;
  // End TFT module connections

  unsigned int current_speed;
  unsigned int pwm_period;
  char txt[24];

  // MCU Initialization
  void MCU_Init()
  {
    GPIO_Digital_Input (&GPIOD_BASE, _GPIO_PINMASK_3 | _GPIO_PINMASK_6); // configure PORTA pins as input
    GPIO_Digital_Output(&GPIOC_ODR, _GPIO_PINMASK_2);

    ADC_Set_Input_Channel(_ADC_CHANNEL_4 | _ADC_CHANNEL_5); // ADC channels 4 and 5
    ADC1_Init(); // ADC Init
    Delay_ms(100);
    current_speed = 5000; //initial value for current duty


    pwm_period = PWM_TIM5_Init(5000);  // PWM Init on 5kHz
    PWM_TIM5_Set_Duty(current_speed,  _PWM_NON_INVERTED, _PWM_CHANNEL1);
    PWM_TIM5_Start(_PWM_CHANNEL1, &_GPIO_MODULE_TIM5_CH1_PA0);
  }
  
    void DrawFrame()
   {
    TFT_Fill_Screen(CL_TEAL);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_TEAL, 0, 0, CL_TEAL, CL_TEAL);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Brushless  Click  Board  Example", 12, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyMx PRO v7 for STM32 ARM", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("PWM  DUTY", 10, 80 );
    TFT_Write_Text("PWM  MAX", 10, 140);
    WordToStr(pwm_period, txt);
    TFT_Write_Text(txt, 150, 140);
    TFT_Set_Pen(CL_TEAL, 1);
  }

  void main() 
  {
    MCU_Init();
    TFT_Init_ILI9341_8bit(320, 240);  // TFT Init
    TFT_BLED = 1;
    DrawFrame();

    MOTOR_DIR = CCW;  // Setting motor direction to counter-clock-wise

    while(1) {

      if (GPIOD_IDR.B3) {                // button on PD3 pressed
        Delay_ms(50);
        current_speed = current_speed + STEP;   // increment current_duty
        if (current_speed > pwm_period)       // if we increase current_duty greater then possible pwm_period value
        {
          current_speed = 0;                    // reset current_duty value to zero
        }
        PWM_TIM5_Set_Duty(current_speed,  _PWM_NON_INVERTED, _PWM_CHANNEL1);  // set newly acquired duty ratio
       }

      if (GPIOD_IDR.B6) {                // button on PD6 pressed
        Delay_ms(50);
        current_speed = current_speed - STEP;
        if (current_speed > pwm_period)       // if we decrease current_duty greater then possible pwm_period value (overflow)
        {
            current_speed = pwm_period;          // set current_duty to value of pwm_period
        }
        PWM_TIM5_Set_Duty(current_speed,  _PWM_NON_INVERTED, _PWM_CHANNEL1); // set newly acquired duty ratio

       }

      Delay_ms(20);

     // current_speed display
      WordToStr(current_speed, txt);
      TFT_Rectangle(150, 80, 250, 110);
      TFT_Write_Text(txt, 150, 80);
    }

  }